/*!
* distribute.it
* Copyright (c) 2011 Peter Sunde <peter.sunde@gmail.com>
* MIT Licensed
*/

/*!
* Module dependencies.
*/
var routeHandler = require('./route.js').routeHandler
    , util = require('../core/util.js')
    , mime = require('mime')
    , fs = require('fs')
    , path = require('path');

/**
* Create a new route controller. 
* 
* @api public
*/
function routeController() {
    
    // list of registered routes
    this.route = {
        get: [],
        post: []
    };
    
    // list of files and which routes they introduced
    this.files = [];
    
    // check if we are locked by setup
    this.setupLock = (function() {
        try {
            fs.statSync(path.join(__dirname, '../../setup.lock'));
            return true;
        } catch(e) { }
        return false;
    })();
    
}

/**
* Check if current route exists or not
* 
* @param {String} name The name of current route
* @param {String} method The request method (get, set etc..)
* @return {Boolean} True if route exists, False otherwise.
* @api public
*/
routeController.prototype.find = function find(name, method) {
    if(this.route[method]) {
        // @TODO: Need to investage express route handling
        // so that we avoid validating a route twice.
        for(var i = 0, l = this.route[method].length; i < l; i++) {
            var lookup = this.route[method][i].lookup;
            if(lookup(name)) return this.route[method][i];
        }
    }
    return false;
}

/**
* Add a route to our routing table
* 
* @param {String} name The name of current route
* @param {Function} fn The route initialize callback
* @return {Boolean} True if route does not exist, False otherwise.
* @api public
*/
routeController.prototype.addRoute = function addRoute(module, global) {
    
    // create a new route container
    var route = new routeHandler(global)
        , self = this;

    // initialize the route setup
    module.apply(route, []);
    
    // loop through route methods
    for(var method in route.method) {
        var methodCallback = route.method[method];
        if(typeof methodCallback === 'function') {
            this.route[method].push(methodCallback);
            (function(route, fn) {
                global.app[method](route.requestUrl, function(req, res, next) {

                   // check if the platform requires setup
                   if(self.setupLock) {
                       if(route.requestUrl !== '/setup') {
                           res.redirect('/setup');
                           return;
                       } else {
                           fn.apply({ route: route, user: {} }, [req,res,next]);
                           return;
                       }
                   } else if(!self.setupLock && route.requestUrl === '/setup') {
                       res.redirect('/404');
                       return;
                   }
                
                   // set the current session data
                   route.session = req.session;
                   
                   // check if user is authorized
                   if(!route.hasSessionAuth()) {
                       global.app.redirect("/login");
                       return;
                   }
                   
                   // check if user has access to this route
                   if(!route.isAccessable()) {
                       global.app.render('/permission');
                       return;
                   }
                    
                   // allow the request to actually happen
                   fn.apply({ route: route, user: {} }, [req,res,next]);
                    
                })
            })(route, methodCallback);
        }
    }
   
}

/**
* Walks recursively through a directory and
* finds all routes present and initializes them.
* 
* @param {String} path Directory to read from
* @api public
*/
routeController.prototype.require = function (path, global) {
    var self = this;
    util.readdir(path, function(err, files) {
      if(err) throw err;
      files.forEach(function(file) {
           if(mime.lookup(file) === 'application/javascript') {
      
              // require the view
              var module = require("../../"+file);

              // add file to list
              self.files.push(file);

              // check for exported methods
              for(var name in module) {
                  self.addRoute(module[name], global);
              }

          } 
      });
    });
}

exports.routeController = routeController;